"""Module for listing directories"""
import commands
import os


class BaseLister(object):
    """Base class for listers"""

    def __init__(self, location, ignore=[]):
        self.location = location
        self.ignore = ignore
        self.contents = sorted(
            [item for item in sorted(self._calculate_contents())
             if item not in self.ignore])

    def _calculate_contents(self):
        """Return list of contents"""
        raise NotImplementedError('Implement in subclass')

    def fullpath(self, item=None):
        """Return full path of item"""
        raise NotImplementedError('Implement in subclass')

    def traverse(self, subdir):
        """Return similar instance for subdir"""
        return self.__class__(self.fullpath(subdir), ignore=self.ignore)

    @property
    def is_project(self):
        """Return whether this location is something with trunk/tags"""
        return 'trunk' in self.contents

    def is_dir(self, item):
        """Return whether item is a directory"""
        raise NotImplementedError('Implement in subclass')

    @property
    def head(self):
        """Return head/trunk when we're a project, if found"""
        if 'trunk' in self.contents:
            return self.traverse('trunk')

    def cat(self, item):
        """Return content of item as lines (assumption: text file)"""
        raise NotImplementedError('Implement in subclass')

    def __cmp__(self, other):
        """Sort by location"""
        if other == None:
            # Special case...
            return -1
        return cmp(self.location, other.location)


class DirLister(BaseLister):
    """List filesystem directories"""

    def _calculate_contents(self):
        return os.listdir(self.location)

    def fullpath(self, item):
        return os.path.join(self.location, item)

    def cat(self, item):
        return open(self.fullpath(item)).read().splitlines()

    def is_dir(self, item):
        return os.path.isdir(self.fullpath(item))


class SvnLister(BaseLister):
    """List svn directory contents"""

    def _calculate_contents(self):
        output = commands.getoutput('svn list %s' % self.location)
        lines = [line.strip() for line in output.splitlines()]
        # Directories are svn-listed as 'directoryname/', so store them first
        # in a list of directories (for is_dir()) and strip the slash
        # afterwards.
        self._directories = [line.rstrip('/') for line in lines
                             if line.endswith('/')]
        lines = [line.rstrip('/') for line in lines]
        return [line for line in lines if line]

    def fullpath(self, item):
        # svn always uses forward slashes.
        return '/'.join([self.location, item])

    def cat(self, item):
        output = commands.getoutput('svn cat %s' % self.fullpath(item))
        return output.splitlines()

    def is_dir(self, item):
        return item in self._directories



import logging


logger = logging.getLogger('tagfinder')


class Finder(object):
    """Recursively detect trunk/tag projects in a structure"""

    def __init__(self, lister, info_extracter,
                 stop_indicators=[]):
        self.lister = lister
        self.stop_indicators = stop_indicators
        projects = sorted(self._find_projects())
        self.projects = [info_extracter(project) for project in projects]

    def _find_projects(self, lister=None):
        """Iterate through the structure and return projects"""
        projects = []
        if lister == None:
            lister = self.lister
        if lister.is_project:
            logger.info("Found project: %s", lister.location)
            projects.append(lister)
        elif [item for item in lister.contents
              if item in self.stop_indicators]:
            logger.debug("Found stop indicator, not doing anything with %s",
                         lister.location)
            return projects
        else:
            subdirs = [subdir for subdir in lister.contents
                       if lister.is_dir(subdir)]
            for subdir in subdirs:
                projects += self._find_projects(
                    lister=lister.traverse(subdir))
        return projects



import commands
import os
import pkg_resources
import shutil
import tempfile

import z3c.testsetup


def svn_setup(test):
    """Set up test svn repository in a temporary directory"""
    test.tempdir = tempfile.mkdtemp()
    repo = os.path.join(test.tempdir, 'repo')
    repo_url = 'file://' + repo
    test.globs['repo_url'] = repo_url
    commands.getoutput('svnadmin create ' + repo)
    fixture_dir = pkg_resources.resource_filename('tha.tagfinder.tests',
                                                  'fixture')
    test.globs['fixture_dir'] = fixture_dir
    export = os.path.join(test.tempdir, 'export')
    output = commands.getoutput('svn export %s %s' % (fixture_dir, export))
    if 'not a working copy' in output:
        # We're not working from a local svn checkout, but for instance from
        # an egg.  In that case, use the fixture dir as the "export"
        # location.
        export = fixture_dir
    commands.getoutput('svn import %s %s -m "import"' % (export, repo_url))


def svn_teardown(test):
    """Clean up temporary directory"""
    #print test.tempdir
    shutil.rmtree(test.tempdir)


test_suite = z3c.testsetup.register_all_tests('tha.tagfinder',
                                              setup=svn_setup,
                                              teardown=svn_teardown)



"""Extracter that extract info from a project"""
from distutils.version import LooseVersion
import re


NAME_PATTERN = re.compile(r"""
    name\W*=\W*    # 'name =  ' with possible whitespace
    [\"\']         # Opening (double) quote.
    (?P<name>.+?)  # Something, but match non-greedy, store in 'name'
    [\"\']         # Closing (double) quote.
    """, re.VERBOSE)


class TagNotFoundError(Exception):
    pass


class BaseExtracter(object):
    """Basic info extracter"""

    def __init__(self, lister):
        """Just return the location info"""
        self.directory = lister
        self._tags = None

    @property
    def location(self):
        """Root of the project (so dir that contains trunk/tags)"""
        return self.directory.location

    @property
    def trunk(self):
        """Trunk location"""
        return self.directory.fullpath('trunk')

    @property
    def tags(self):
        if self._tags is None:
            self._tags = self._calculate_tags()
        return self._tags

    def _calculate_tags(self):
        """Return dict of tag/directory"""
        if 'tags' not in self.directory.contents:
            return []
        tagdir = self.directory.traverse('tags')
        tags = tagdir.contents
        # Filter out everything not starting with a number
        tags = [tag for tag in tags
                if isinstance(LooseVersion(tag).version[0], int)]
        # Not handled yet: tags that don't match setup.py.
        # Is that still needed?
        tags.sort(key=version_key)
        return tags

    def tag_location(self, tag):
        """Return location of tag"""
        if 'tags' not in self.directory.contents:
            raise TagNotFoundError('Tag dir not found')
        tagdir = self.directory.traverse('tags')
        if tag not in tagdir.contents:
            raise TagNotFoundError(tag)
        return tagdir.fullpath(tag)

    @property
    def name(self):
        return self._setup_py_name() or self._base_name()

    def _base_name(self):
        """Return last path element as name"""
        return self.location.split('/')[-1]

    def _setup_py_name(self):
        """Return name according to the setup.py, if available"""
        if 'setup.py' in self.directory.head.contents:
            for line in self.directory.head.cat('setup.py'):
                match = NAME_PATTERN.search(line)
                if match:
                    name = match.group('name').strip()
                    return name


def version_key(tag_string):
    """Return version value suitable for sorting"""
    return LooseVersion(tag_string).version




from setuptools import setup, find_packages
import os.path

version = '1.1'

long_description = '\n\n'.join([
    open('README.txt').read(),
    open(os.path.join('src', 'tha', 'tagfinder', 'USAGE.txt')).read(),
    open('TODO.txt').read(),
    open('CHANGES.txt').read(),
    ])

setup(name='tha.tagfinder',
      version=version,
      description="",
      long_description=long_description,
      # Get strings from http://www.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[],
      keywords=[],
      author='The Health Agency',
      author_email='techniek@thehealthagency.com',
      url='http://www.thehealthagency.com',
      license='BSD',
      package_dir={'': 'src'},
      packages=find_packages('src'),
      namespace_packages=['tha'],
      include_package_data=True,
      zip_safe=False,
      install_requires=['setuptools',
                        # Add extra requirements here
                        ],
      extras_require = {
          'test': ['z3c.testsetup>=0.3'],
          },
      entry_points={
          'console_scripts': [
          ]},
      )
